var _instance;
export var MockFetcherManager = /*#__PURE__*/function () {
  function MockFetcherManager(FetchFetcher, _ref) {
    var _this = this;
    var InlineError = _ref.InlineError;
    this._fakeRequests = void 0;
    this._methods = ['get', 'patch', 'post'];
    this.FetchFetcher = FetchFetcher;
    this._originalsFetcher = this._methods.map(function (method) {
      return _this.FetchFetcher.prototype[method];
    });
    this._fakeRequests = {};
    this._hasInlineErrors = InlineError;
    this._methods.forEach(function (method) {
      _this.FetchFetcher.prototype[method] = _this._request(method);
    });
  }
  MockFetcherManager.create = function create(FetchFetcher, _ref2) {
    var InlineError = _ref2.InlineError;
    if (_instance) return _instance;
    _instance = new MockFetcherManager(FetchFetcher, {
      InlineError: InlineError
    });
    return _instance;
  };
  MockFetcherManager.restore = function restore() {
    if (!_instance) return console.warn('Unable to restore a non-initialized MockFetcherManager');
    _instance.restore();
    _instance = undefined;
  };
  var _proto = MockFetcherManager.prototype;
  _proto.addMock = function addMock(_ref3) {
    var url = _ref3.url,
      method = _ref3.method,
      error = _ref3.error,
      mock = _ref3.mock,
      force = _ref3.force;
    var key = method.toUpperCase() + '::' + url;
    if (!force && this._fakeRequests[key]) throw new Error("[MockFetcherManager#addMock] forbidden overwrite the request " + key);
    this._fakeRequests[key] = [error, mock];
  };
  _proto.validate = function validate(_ref4) {
    var url = _ref4.url,
      method = _ref4.method;
    var key = method.toUpperCase() + '::' + url;
    if (this._fakeRequests[key]) throw new Error("[MockFetcherManager#validate] request " + key + " don't consume");
  };
  _proto.restore = function restore() {
    var _this2 = this;
    if (Object.keys(this._fakeRequests).length === 0) {
      this._methods.forEach(function (method, index) {
        return _this2.FetchFetcher.prototype[method] = _this2._originalsFetcher[index];
      });
    } else {
      throw new Error("[MockFetcherManager#restore]\n        Unabled restore the FetchFetcher because there are request w/out been consume\n        - Dont match any mock:\n        " + Object.keys(this._fakeRequests).join('\n\t\t') + "\n      ");
    }
  };
  _proto._request = function _request(method) {
    var self = this;
    return function () {
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      var url = args[0];
      var requestKey = method.toUpperCase() + '::' + url;
      // this === FetchFetcher instance
      if (self._fakeRequests[requestKey]) {
        var _self$_fakeRequests$r = self._fakeRequests[requestKey],
          error = _self$_fakeRequests$r[0],
          response = _self$_fakeRequests$r[1];
        delete self._fakeRequests[requestKey];
        if (!self._hasInlineErrors) return error ? Promise.reject(error) : Promise.resolve({
          data: response
        });
        return Promise.resolve([error, response]);
      } else {
        console.warn("[MockFetcherManager#_request]\n        - Request make " + requestKey + "\n        - Dont match any mock:\n        " + Object.keys(self._fakeRequests).join('\n\t\t') + "\n      ");
      }
      return this._originalsFetcher[method].apply(this, args);
    };
  };
  return MockFetcherManager;
}();